# -*- coding: utf-8 -*-

"""
####################################################################
#                                                                  #
#                           AWI Basemap                            #
#       PROCESS GENERAL BATHYMETRIC CHART OF THE OCEANS DATA       #
#                                                                  #
####################################################################

This script is processing the General Bathymetric Chart of the
Oceans (GEBCO) input for the AWI Basemap. The global map is
downsampled to the target resolution (RESOLUTION_DEG) set in the
configuration. Then the global grid is clipped to the Arctic and
Antarctic extent, reprojected (warped) to the Arctic and Antarctic
projections (EPSG_ARCTIC and EPSG_ANTARCTIC), and downsampled to the
target resolution (RESOLUTION_M).

"""


#=============================================================================
#    SCRIPT INFO
#=============================================================================

__author__ = 'Simon Dreutter'
__version__ = '0.1'
__date__ = '2020-12-17'
__email__ = 'simon.dreutter@awi.de'
__status__ = 'Developement'


#=============================================================================
#    IMPORT
#=============================================================================

# import AWI Basemap configuration
from config import *


#=============================================================================
#    PROCESS
#=============================================================================

msg = 'Resampling GEBCO...'
cmd = f'gdalwarp {WARP_OPTIONS} -s_srs {EPSG_WORLD} -t_srs {EPSG_WORLD} -tr {RESOLUTION_DEG} {RESOLUTION_DEG} -r cubic -of GTiff {GEBCO_IN} {GEBCO_WORLD}'
# {GDAL_CREATE_OPTIONS}
#run(msg,cmd)

msg = 'Clipping GEBCO for the Arctic...'
cmd = f'gdal_translate -projwin -180.0 90.0 180.0 {ARCTIC_EXTENT_LAT} -of VRT {GEBCO_IN} {GEBCO_ARCTIC_CLIP_VRT}'
run(msg,cmd)

msg = 'Reprojecting GEBCO for the Arctic...'
cmd = f'gdalwarp -s_srs {EPSG_WORLD} -t_srs {EPSG_ARCTIC} -r near -of VRT {GEBCO_ARCTIC_CLIP_VRT} {GEBCO_ARCTIC_FULL_VRT}'
#{WARP_OPTIONS}
run(msg,cmd)

msg = 'Resampling Arctic to target resolution...'
cmd = f'gdalwarp {WARP_OPTIONS} -s_srs {EPSG_ARCTIC} -t_srs {EPSG_ARCTIC} -tr {RESOLUTION_M} {RESOLUTION_M} -r cubic -of GTiff {GEBCO_ARCTIC_FULL_VRT} {TMP_POLAR}'
#{GEBCO_ARCTIC}
run(msg,cmd)

# TEMPORARY
msg = 'Filling North Pole gap...'
cmd = f'{PYTHON} -m gdal_fillnodata -md {INTERPOLATION} -b 1 -of GTiff {TMP_POLAR} {GEBCO_ARCTIC}'
run(msg,cmd)
cleanup(TMP_POLAR)
# /TEMPORARY

msg = 'Clipping GEBCO for the Antarctic...'
cmd = f'gdal_translate -projwin -180.0 {ANTARCTIC_EXTENT_LAT} 180.0 -90.0 -of VRT {GEBCO_IN} {GEBCO_ANTARCTIC_CLIP_VRT}'
run(msg,cmd)

msg = 'Reprojecting GEBCO for the Antarctic...'
cmd = f'gdalwarp -s_srs {EPSG_WORLD} -t_srs {EPSG_ANTARCTIC} -r near -of VRT {GEBCO_ANTARCTIC_CLIP_VRT} {GEBCO_ANTARCTIC_FULL_VRT}'
#{WARP_OPTIONS}
run(msg,cmd)

msg = 'Resampling Antarctic to target resolution...'
cmd = f'gdalwarp {WARP_OPTIONS} -s_srs {EPSG_ANTARCTIC} -t_srs {EPSG_ANTARCTIC} -tr {RESOLUTION_M} {RESOLUTION_M} -r cubic -of GTiff {GEBCO_ANTARCTIC_FULL_VRT} {TMP_POLAR}'
#{GEBCO_ANTARCTIC}
run(msg,cmd)

# TEMPORARY
msg = 'Filling South Pole gap...'
cmd = f'{PYTHON} -m gdal_fillnodata -md {INTERPOLATION} -b 1 -of GTiff {TMP_POLAR} {GEBCO_ANTARCTIC}'
run(msg,cmd)
cleanup(TMP_POLAR)
# /TEMPORARY

cleanup(GEBCO_ARCTIC_CLIP_VRT)
cleanup(GEBCO_ARCTIC_FULL_VRT)
cleanup(GEBCO_ANTARCTIC_CLIP_VRT)
cleanup(GEBCO_ANTARCTIC_FULL_VRT)


#=============================================================================
#    NOTES / TODO
#=============================================================================

"""
The current solution for creating the polar stereographic GEBCO grids (VRT)
produces holes on the north and south pole. Another solution needs to be
found. Normal clip and warp however produces issues:

- Nearest Neighbor: raster artifacts
- Cubic (and others): dateline blur

What to do?

- get rid of interpolation (because of the ugly extrapolation)
"""