# -*- coding: utf-8 -*-

"""
####################################################################
#                                                                  #
#                           AWI Basemap                            #
#                         CREATE BASEMAPS                          #
#                                                                  #
####################################################################

This script creates the final AWI Basemaps in all three projections
and with different color palettes. This happens in two setps. First
the DEM is rendered with the color palette into a RGB color raster.
Then the shaded map is calculated by combining the colored raster
with the shading raster.
At the end, the raster files are finalized by creating tiles and
pyramids.

"""


#=============================================================================
#    SCRIPT INFO
#=============================================================================

__author__ = 'Simon Dreutter'
__version__ = '0.1'
__date__ = '2020-12-17'
__email__ = 'simon.dreutter@awi.de'
__status__ = 'Developement'


#=============================================================================
#    IMPORT
#=============================================================================

# import AWI Basemap configuration
from config import *


#=============================================================================
#    FUNCTIONS
#=============================================================================

def create_basemap(map_params):
    name,gebco,hs,cp,epsg,output = map_params
    msg = f'Creating {name} RGB...'
    cmd = f'gdaldem color-relief {gebco} {cp} {TMP_RGB} -of GTiff -b 1 -compute_edges {GDAL_CREATE_OPTIONS}'
    run(msg,cmd)
    
    msg = f'Creating shaded {name} map...'
    cmd = f'{PYTHON} -m gdal_calc --calc "uint8(((A/255.)*(((B*0.75)+(255.*0.25))/255.))*255)" --format GTiff --type Byte -A {TMP_RGB} --A_band 1 -B {hs} --B_band 1 --allBands=A --overwrite --outfile {TMP_SHADED}'
    # {GDAL_CREATE_OPTIONS_PY}
    run(msg,cmd)
    
    cleanup(TMP_RGB)
    
    msg = f'Finalizing AWI Basemap {name}...'
    cmd = f'gdal_translate -a_srs {epsg} -co TILED=YES -co BLOCKXSIZE=512 -co BLOCKYSIZE=512 -co COPY_SRC_OVERVIEWS=FALSE -co COMPRESS=NONE -stats -of GTiff {TMP_SHADED} {output}'
    run(msg,cmd)
    
    cleanup(TMP_SHADED)
    
    msg = f'Creating pyramids for AWI Basemap {name}...'
    cmd = f'gdaladdo -r {OVERVIEW_METHOD} --config COMPRESS_OVERVIEW NONE --config INTERLEAVE_OVERVIEW PIXEL --config BIGTIFF_OVERVIEW IF_NEEDED --config GDAL_TIFF_OVR_BLOCKSIZE 512 {output} 2 4 8 16 32 64'
    run(msg,cmd)


#=============================================================================
#    PROCESS
#=============================================================================

maps = [
    ('World [default]',     GEBCO_WORLD,     GEBCO_WORLD_HILLSHADE,     CP_DEFAULT, EPSG_WORLD,     BASEMAP_WORLD            ),
    ('Arctic [default]',    GEBCO_ARCTIC,    GEBCO_ARCTIC_HILLSHADE,    CP_DEFAULT, EPSG_ARCTIC,    BASEMAP_ARCTIC           ),
    ('Antarctic [default]', GEBCO_ANTARCTIC, GEBCO_ANTARCTIC_HILLSHADE, CP_DEFAULT, EPSG_ANTARCTIC, BASEMAP_ANTARCTIC        ),
    ('World [bright]',      GEBCO_WORLD,     GEBCO_WORLD_HILLSHADE,     CP_BRIGHT,  EPSG_WORLD,     BASEMAP_WORLD_BRIGHT     ),
    ('Arctic [bright]',     GEBCO_ARCTIC,    GEBCO_ARCTIC_HILLSHADE,    CP_BRIGHT,  EPSG_ARCTIC,    BASEMAP_ARCTIC_BRIGHT    ),
    ('Antarctic [bright]',  GEBCO_ANTARCTIC, GEBCO_ANTARCTIC_HILLSHADE, CP_BRIGHT,  EPSG_ANTARCTIC, BASEMAP_ANTARCTIC_BRIGHT ),
    ('World [dark]',        GEBCO_WORLD,     GEBCO_WORLD_HILLSHADE,     CP_DARK,    EPSG_WORLD,     BASEMAP_WORLD_DARK       ),
    ('Arctic [dark]',       GEBCO_ARCTIC,    GEBCO_ARCTIC_HILLSHADE,    CP_DARK,    EPSG_ARCTIC,    BASEMAP_ARCTIC_DARK      ),
    ('Antarctic [dark]',    GEBCO_ANTARCTIC, GEBCO_ANTARCTIC_HILLSHADE, CP_DARK,    EPSG_ANTARCTIC, BASEMAP_ANTARCTIC_DARK   )
]

for map_params in maps:
    create_basemap(map_params)

cleanup(GEBCO_WORLD)
cleanup(GEBCO_WORLD_HILLSHADE)
cleanup(GEBCO_ARCTIC)
cleanup(GEBCO_ARCTIC_HILLSHADE)
cleanup(GEBCO_ANTARCTIC)
cleanup(GEBCO_ANTARCTIC_HILLSHADE)


#=============================================================================
#    TODO
#=============================================================================

"""
Metadata
"""

#=============================================================================
#    BACKUP
#=============================================================================

"""
msg = 'Creating World RGB...'
cmd = f'gdaldem color-relief {GEBCO_WORLD} {CP_DEFAULT} {GEBCO_WORLD_RGB} -of GTiff -b 1 -compute_edges {GDAL_CREATE_OPTIONS}')
run(msg,cmd)

msg = 'Creating AWI Basemap World...'
cmd = f'python3 -m gdal_calc --calc "uint8(((A/255.)*(((B*0.75)+(255.*0.25))/255.))*255)" --format GTiff --type Byte -A {GEBCO_WORLD_RGB} --A_band 1 -B {GEBCO_WORLD_HILLSHADE} --B_band 1 {GDAL_CREATE_OPTIONS_PY} --allBands=A --overwrite --outfile {BASEMAP_WORLD}')
run(msg,cmd)

msg = 'Creating Arctic RGB...'
cmd = f'gdaldem color-relief {GEBCO_ARCTIC} {CP_DEFAULT} {GEBCO_ARCTIC_RGB} -of GTiff -b 1 -compute_edges {GDAL_CREATE_OPTIONS}')
run(msg,cmd)

msg = 'Creating AWI Basemap Arctic...'
cmd = f'python3 -m gdal_calc --calc "uint8(((A/255.)*(((B*0.75)+(255.*0.25))/255.))*255)" --format GTiff --type Byte -A {GEBCO_ARCTIC_RGB} --A_band 1 -B {GEBCO_ARCTIC_HILLSHADE} --B_band 1 {GDAL_CREATE_OPTIONS_PY} --allBands=A --overwrite --outfile {BASEMAP_ARCTIC}')
run(msg,cmd)

msg = 'Creating Antarctic RGB...'
cmd = f'gdaldem color-relief {GEBCO_ANTARCTIC} {CP_DEFAULT} {GEBCO_ANTARCTIC_RGB} -of GTiff -b 1 -compute_edges {GDAL_CREATE_OPTIONS}')
run(msg,cmd)

msg = 'Creating AWI Basemap Antarctic...'
cmd = f'python3 -m gdal_calc --calc "uint8(((A/255.)*(((B*0.75)+(255.*0.25))/255.))*255)" --format GTiff --type Byte -A {GEBCO_ANTARCTIC_RGB} --A_band 1 -B {GEBCO_ANTARCTIC_HILLSHADE} --B_band 1 {GDAL_CREATE_OPTIONS_PY} --allBands=A --overwrite --outfile {BASEMAP_ANTARCTIC}')
run(msg,cmd)
"""
